import AddressBook
import Cocoa
import objc


class ABPerson(objc.Category(AddressBook.ABPerson)):
    # Pull first and last name, organization, and record flags
    # If the entry is a company, display the organization name instead
    def displayName(self):
        firstName = self.valueForProperty_(AddressBook.kABFirstNameProperty)
        lastName = self.valueForProperty_(AddressBook.kABLastNameProperty)
        companyName = self.valueForProperty_(AddressBook.kABOrganizationProperty)
        flags = self.valueForProperty_(AddressBook.kABPersonFlags)
        if flags is None:
            flags = 0

        if (flags & AddressBook.kABShowAsMask) == AddressBook.kABShowAsCompany:
            if len(companyName):
                return companyName

        lastNameFirst = (
            flags & AddressBook.kABNameOrderingMask
        ) == AddressBook.kABLastNameFirst
        hasFirstName = firstName is not None
        hasLastName = lastName is not None

        if hasLastName and hasFirstName:
            if lastNameFirst:
                return Cocoa.NSString.stringWithString_("%s %s" % (lastName, firstName))
            else:
                return Cocoa.NSString.stringWithString_("%s %s" % (firstName, lastName))

        if hasLastName:
            return lastName

        return firstName

    def compareDisplayNames_(self, person):
        return self.displayName().localizedCaseInsensitiveCompare_(person.displayName())



import InstantMessage
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestIMService(TestCase):
    def testConstants(self):
        self.assertIsInstance(InstantMessage.IMServiceStatusChangedNotification, str)
        self.assertIsInstance(InstantMessage.IMMyStatusChangedNotification, str)
        self.assertIsInstance(InstantMessage.IMPersonStatusChangedNotification, str)
        self.assertIsInstance(InstantMessage.IMPersonInfoChangedNotification, str)
        self.assertIsInstance(
            InstantMessage.IMStatusImagesChangedAppearanceNotification, str
        )

        self.assertEqual(InstantMessage.IMPersonStatusUnknown, 0)
        self.assertEqual(InstantMessage.IMPersonStatusOffline, 1)
        self.assertEqual(InstantMessage.IMPersonStatusIdle, 2)
        self.assertEqual(InstantMessage.IMPersonStatusAway, 3)
        self.assertEqual(InstantMessage.IMPersonStatusAvailable, 4)
        self.assertEqual(InstantMessage.IMPersonStatusNoStatus, 5)

        self.assertEqual(InstantMessage.IMServiceStatusLoggedOut, 0)
        self.assertEqual(InstantMessage.IMServiceStatusDisconnected, 1)
        self.assertEqual(InstantMessage.IMServiceStatusLoggingOut, 2)
        self.assertEqual(InstantMessage.IMServiceStatusLoggingIn, 3)
        self.assertEqual(InstantMessage.IMServiceStatusLoggedIn, 4)

        self.assertIsInstance(InstantMessage.IMPersonServiceNameKey, str)
        self.assertIsInstance(InstantMessage.IMPersonScreenNameKey, str)
        self.assertIsInstance(InstantMessage.IMPersonStatusKey, str)
        self.assertIsInstance(InstantMessage.IMPersonStatusMessageKey, str)
        self.assertIsInstance(InstantMessage.IMPersonIdleSinceKey, str)
        self.assertIsInstance(InstantMessage.IMPersonFirstNameKey, str)
        self.assertIsInstance(InstantMessage.IMPersonLastNameKey, str)
        self.assertIsInstance(InstantMessage.IMPersonEmailKey, str)
        self.assertIsInstance(InstantMessage.IMPersonPictureDataKey, str)
        self.assertIsInstance(InstantMessage.IMPersonAVBusyKey, str)
        self.assertIsInstance(InstantMessage.IMPersonCapabilitiesKey, str)
        self.assertIsInstance(InstantMessage.IMCapabilityText, str)
        self.assertIsInstance(InstantMessage.IMCapabilityDirectIM, str)
        self.assertIsInstance(InstantMessage.IMCapabilityFileTransfer, str)
        self.assertIsInstance(InstantMessage.IMCapabilityFileSharing, str)
        self.assertIsInstance(InstantMessage.IMCapabilityAudioConference, str)
        self.assertIsInstance(InstantMessage.IMCapabilityVideoConference, str)

    @min_os_level("10.5")
    def testFunctions10_5(self):
        v = InstantMessage.IMComparePersonStatus(
            InstantMessage.IMPersonStatusOffline, InstantMessage.IMPersonStatusAway
        )
        self.assertIsInstance(v, int)



import AddressBook  # noqa: F401
import Cocoa
import InstantMessage

kAddressBookPersonStatusChanged = "AddressBookPersonStatusChanged"
kStatusImagesChanged = "StatusImagesChanged"


class ServiceWatcher(Cocoa.NSObject):
    def startMonitoring(self):
        nCenter = InstantMessage.IMService.notificationCenter()
        if nCenter is None:
            return None

        nCenter.addObserver_selector_name_object_(
            self,
            b"imPersonStatusChangedNotification:",
            InstantMessage.IMPersonStatusChangedNotification,
            None,
        )

        nCenter.addObserver_selector_name_object_(
            self,
            b"imStatusImagesChangedAppearanceNotification:",
            InstantMessage.IMStatusImagesChangedAppearanceNotification,
            None,
        )

    def stopMonitoring(self):
        nCenter = InstantMessage.IMService.notificationCenter()
        nCenter.removeObserver_(self)

    def awakeFromNib(self):
        self.startMonitoring()

    # Received from IMService's custom notification center. Posted when a
    # different user (screenName) logs in, logs off, goes away,
    # and so on. This notification is for the IMService object. The user
    # information dictionary will always contain an
    # IMPersonScreenNameKey and an IMPersonStatusKey, and no others.
    def imPersonStatusChangedNotification_(self, notification):
        service = notification.object()
        userInfo = notification.userInfo()
        screenName = userInfo[InstantMessage.IMPersonScreenNameKey]
        abPersons = service.peopleWithScreenName_(screenName)

        center = Cocoa.NSNotificationCenter.defaultCenter()
        for person in abPersons:
            center.postNotificationName_object_(kAddressBookPersonStatusChanged, person)

    # Received from IMService's custom notification center. Posted when the
    # user changes their preferred images for displaying status.
    # This notification is relevant to no particular object. The user
    # information dictionary will not contain keys. Clients that display
    # status information graphically (using the green/yellow/red dots) should
    # call <tt>imageURLForStatus:</tt> to get the new image.
    # See "Class Methods" for IMService in this document.
    def imStatusImagesChangedAppearanceNotification_(self, notification):
        Cocoa.NSNotificationCenter.defaultCenter().postNotificationName_object_(
            kStatusImagesChanged, self
        )



# This file is generated by objective.metadata
#
# Last update: Sat Jun 27 17:33:22 2020
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$IMAVManagerStateChangedNotification$IMAVManagerURLToShareChangedNotification$IMCapabilityAudioConference$IMCapabilityDirectIM$IMCapabilityFileSharing$IMCapabilityFileTransfer$IMCapabilityText$IMCapabilityVideoConference$IMMyStatusChangedNotification$IMPersonAVBusyKey$IMPersonCapabilitiesKey$IMPersonEmailKey$IMPersonFirstNameKey$IMPersonIdleSinceKey$IMPersonInfoChangedNotification$IMPersonLastNameKey$IMPersonPictureDataKey$IMPersonScreenNameKey$IMPersonServiceNameKey$IMPersonStatusChangedNotification$IMPersonStatusKey$IMPersonStatusMessageKey$IMServiceStatusChangedNotification$IMStatusImagesChangedAppearanceNotification$"""
enums = """$IMAVInactive@0$IMAVPending@4$IMAVRequested@1$IMAVRunning@5$IMAVShuttingDown@2$IMAVStartingUp@3$IMPersonStatusAvailable@4$IMPersonStatusAway@3$IMPersonStatusIdle@2$IMPersonStatusNoStatus@5$IMPersonStatusOffline@1$IMPersonStatusUnknown@0$IMServiceStatusDisconnected@1$IMServiceStatusLoggedIn@4$IMServiceStatusLoggedOut@0$IMServiceStatusLoggingIn@3$IMServiceStatusLoggingOut@2$IMVideoOptimizationDefault@0$IMVideoOptimizationReplacement@2$IMVideoOptimizationStills@1$"""
misc.update({})
functions = {"IMComparePersonStatus": (b"qQQ",)}
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"IMAVControl", b"isEnabled", {"retval": {"type": b"Z"}})
    r(b"IMAVControl", b"setAction:", {"arguments": {2: {"sel_of_type": b"v@:@"}}})
    r(b"IMAVControl", b"setEnabled:", {"arguments": {2: {"type": b"Z"}}})
    r(
        b"NSObject",
        b"getOpenGLBufferContext:pixelFormat:",
        {
            "retval": {"type": b"v"},
            "arguments": {
                2: {"type": b"^^{_CGLContextObject=}", "type_modifier": b"o"},
                3: {"type": b"^^{_CGLPixelFormatObject=}", "type_modifier": b"o"},
            },
        },
    )
    r(
        b"NSObject",
        b"getPixelBufferPixelFormat:",
        {
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"^I", "type_modifier": b"o"}},
        },
    )
    r(
        b"NSObject",
        b"renderIntoOpenGLBuffer:onScreen:forTime:",
        {
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": "^{__CVBuffer=}"},
                3: {"type": b"^i", "type_modifier": b"n"},
                4: {
                    "type": b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                    "type_modifier": b"n",
                },
            },
        },
    )
    r(
        b"NSObject",
        b"renderIntoPixelBuffer:forTime:",
        {
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": "^{__CVBuffer=}"},
                3: {
                    "type": b"^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                    "type_modifier": b"n",
                },
            },
        },
    )
finally:
    objc._updatingMetadata(False)
protocols = {
    "IMVideoDataSource": objc.informal_protocol(
        "IMVideoDataSource",
        [
            objc.selector(
                None,
                b"renderIntoPixelBuffer:forTime:",
                b"Z@:^{__CVBuffer=}^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                isRequired=False,
            ),
            objc.selector(
                None, b"getPixelBufferPixelFormat:", b"v@:^I", isRequired=False
            ),
            objc.selector(
                None,
                b"getOpenGLBufferContext:pixelFormat:",
                b"v@:^^{_CGLContextObject=}^^{_CGLPixelFormatObject=}",
                isRequired=False,
            ),
            objc.selector(
                None,
                b"renderIntoOpenGLBuffer:onScreen:forTime:",
                b"Z@:^{__CVBuffer=}^i^{_CVTimeStamp=IiqQdq{CVSMPTETime=ssIIIssss}QQ}",
                isRequired=False,
            ),
        ],
    )
}
expressions = {}

# END OF FILE



import InstantMessage
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestIMAVManagerHelper(InstantMessage.NSObject):
    def getPixelBufferPixelFormat_(self, _):
        return 10

    def renderIntoPixelBuffer_forTime_(self, buf, time):
        return False

    def getOpenGLBufferContext_pixelFormat_(self, ctx, fmt):
        pass

    def renderIntoOpenGLBuffer_onScreen_forTime_(self, buffer, screen, time):
        return False


class TestIMAVManager(TestCase):
    @min_os_level("10.5")
    def testConstants(self):
        self.assertIsInstance(InstantMessage.IMAVManagerStateChangedNotification, str)
        self.assertIsInstance(
            InstantMessage.IMAVManagerURLToShareChangedNotification, str
        )

        self.assertEqual(InstantMessage.IMAVInactive, 0)
        self.assertEqual(InstantMessage.IMAVRequested, 1)
        self.assertEqual(InstantMessage.IMAVShuttingDown, 2)
        self.assertEqual(InstantMessage.IMAVStartingUp, 3)
        self.assertEqual(InstantMessage.IMAVPending, 4)
        self.assertEqual(InstantMessage.IMAVRunning, 5)

        self.assertEqual(InstantMessage.IMVideoOptimizationDefault, 0)
        self.assertEqual(InstantMessage.IMVideoOptimizationStills, 1 << 0)
        self.assertEqual(InstantMessage.IMVideoOptimizationReplacement, 1 << 1)

    def testInformalProtocol(self):
        # self.assert_( hasattr(protocols, 'IMVideoDataSource') )
        # self.assert_( isinstance(protocols.IMVideoDataSource, objc.informal_protocol) )

        self.assertArgIsOut(TestIMAVManagerHelper.getPixelBufferPixelFormat_, 0)
        self.assertArgHasType(
            TestIMAVManagerHelper.getPixelBufferPixelFormat_, 0, b"o^" + objc._C_UINT
        )

        self.assertResultIsBOOL(TestIMAVManagerHelper.renderIntoPixelBuffer_forTime_)
        self.assertArgIsIn(TestIMAVManagerHelper.renderIntoPixelBuffer_forTime_, 1)

        self.assertResultIsBOOL(
            TestIMAVManagerHelper.renderIntoOpenGLBuffer_onScreen_forTime_
        )



import AddressBook
import Cocoa
import InstantMessage
import objc
from ServiceWatcher import kAddressBookPersonStatusChanged, kStatusImagesChanged


class PeopleDataSource(Cocoa.NSObject):
    _abPeople = objc.ivar()
    _imPersonStatus = objc.ivar()  # Parallel array to abPeople
    _table = objc.IBOutlet()

    def awakeFromNib(self):
        self._imPersonStatus = Cocoa.NSMutableArray.alloc().init()

        # We don't need to query the staus of everyone, we will wait for
        # notifications of their status to arrive, so we just set them all up
        # as offline.
        self.setupABPeople()

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self,
            b"abDatabaseChangedExternallyNotification:",
            AddressBook.kABDatabaseChangedExternallyNotification,
            None,
        )

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self,
            b"addressBookPersonStatusChanged:",
            kAddressBookPersonStatusChanged,
            None,
        )

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, b"statusImagesChanged:", kStatusImagesChanged, None
        )

    # This dumps all the status information and rebuilds the array against
    # the current _abPeople
    # Fairly expensive, so this is only done when necessary
    def rebuildStatusInformation(self):
        # Now scan through all the people, adding their status to the status
        # cache array
        for i, person in enumerate(self._abPeople):
            # Let's assume they're offline to start
            bestStatus = InstantMessage.IMPersonStatusOffline

            for service in InstantMessage.IMService.allServices():
                screenNames = service.screenNamesForPerson_(person)

                for screenName in screenNames:
                    dictionary = service.infoForScreenName_(screenName)
                    status = dictionary.get(InstantMessage.IMPersonStatusKey)
                    if status is not None:
                        thisStatus = status
                        if (
                            InstantMessage.IMComparePersonStatus(bestStatus, thisStatus)
                            != Cocoa.NSOrderedAscending
                        ):
                            bestStatus = thisStatus

            self._imPersonStatus[i] = bestStatus

        self._table.reloadData()

    # Rebuild status information for a given person, much faster than a full
    # rebuild
    def rebuildStatusInformationForPerson_(self, forPerson):
        for i, person in enumerate(self._abPeople):
            if person is forPerson:
                bestStatus = InstantMessage.IMPersonStatusOffline

                # Scan through all the services, taking the 'best' status we
                # can find
                for service in InstantMessage.IMService.allServices():
                    screenNames = service.screenNamesForPerson_(person)

                    # Ask for the status on each of their screen names
                    for screenName in screenNames:
                        dictionary = service.infoForScreenName_(screenName)
                        status = dictionary.get(InstantMessage.IMPersonStatusKey)
                        if status is not None:
                            thisStatus = status
                            if (
                                InstantMessage.IMComparePersonStatus(
                                    bestStatus, thisStatus
                                )
                                != Cocoa.NSOrderedAscending
                            ):
                                bestStatus = thisStatus

                self._imPersonStatus[i] = bestStatus
                self._table.reloadData()
                break

    # Sets up all our internal data
    def setupABPeople(self):
        # Keep around a copy of all the people in the AB now
        self._abPeople = (
            AddressBook.ABAddressBook.sharedAddressBook().people().mutableCopy()
        )

        # Sort them by display name
        self._abPeople.sortUsingSelector_("compareDisplayNames:")

        # Assume everyone is offline.
        self._imPersonStatus.removeAllObjects()
        offlineNumber = InstantMessage.IMPersonStatusOffline
        for _ in range(len(self._abPeople)):
            self._imPersonStatus.append(offlineNumber)

    # This will do a full flush of people in our AB Cache, along with
    # rebuilding their status */
    def reloadABPeople(self):
        self.setupABPeople()

        # Now recache all the status info, this will spawn a reload of the table
        self.rebuildStatusInformation()

    def numberOfRowsInTableView_(self, tableView):
        if self._abPeople is None:
            return 0
        return len(self._abPeople)

    def tableView_objectValueForTableColumn_row_(self, tableView, tableColumn, row):
        identifier = tableColumn.identifier()
        if identifier == "image":
            status = self._imPersonStatus[row]
            return Cocoa.NSImage.imageNamed_(
                InstantMessage.IMService.imageNameForStatus_(status)
            )

        elif identifier == "name":
            return self._abPeople[row].displayName()

        return None

    # Posted from ServiceWatcher
    # The object of this notification is an ABPerson who's status has
    # Changed
    def addressBookPersonStatusChanged_(self, notification):
        self.rebuildStatusInformationForPerson_(notification.object())

    # Posted from ServiceWatcher
    # We should reload the tableview, because the user has changed the
    # status images that iChat is using.
    def statusImagesChanged_(self, notification):
        self._table.reloadData()

    # If the AB database changes, force a reload of everyone
    # We could look in the notification to catch differential updates, but
    # for now this is fine.
    def abDatabaseChangedExternallyNotification_(self, notification):
        self.reloadABPeople()



"""
Python mapping for the InstantMessage framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import Foundation
import objc
import Quartz
from InstantMessage import _metadata

sys.modules["InstantMessage"] = mod = objc.ObjCLazyModule(
    "InstantMessage",
    "com.apple.IMFramework",
    objc.pathForFramework("/System/Library/Frameworks/InstantMessage.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (Foundation, Quartz),
)


del sys.modules["InstantMessage._metadata"]



"""
Wrappers for "InstantMessage" framework on macOS 10.5 or later. This framework
allows you to access iChat information, as well as a way to provide an
auxilliary video source to iChat Theater.

These wrappers don't include documentation, please check Apple's documention
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks

Note that this framework is deprecated in OSX 10.9, use the Social framework
instead if you target that OSX release.
"""
from pyobjc_setup import setup

VERSION = '7.2'

setup(
    name="pyobjc-framework-InstantMessage",
    description="Wrappers for the framework InstantMessage on macOS",
    min_os_level="10.5",
    packages=["InstantMessage"],
    version=VERSION,
    install_requires=[
        "pyobjc-core>=" + VERSION,
        "pyobjc-framework-Cocoa>=" + VERSION,
        "pyobjc-framework-Quartz>=" + VERSION,
    ],
    long_description=__doc__,
)



import InstantMessage
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestIMControl(TestCase):
    @min_os_level("10.6")
    def testMethods(self):
        self.assertArgIsSEL(InstantMessage.IMAVControl.setAction_, 0, b"v@:@")
        self.assertResultIsBOOL(InstantMessage.IMAVControl.isEnabled)
        self.assertArgIsBOOL(InstantMessage.IMAVControl.setEnabled_, 0)
